# Import libraries
import xml.etree.ElementTree as ET
import os
from zipfile import ZipFile

from com.inductiveautomation.ignition.gateway import IgnitionGateway
context = IgnitionGateway.get()
dataDirectory = str(context.getSystemManager().getDataDir().getAbsolutePath())
tempFolder = os.path.join(os.path.dirname(dataDirectory), 'temp/iconImport')
perspectiveIconFolder = os.path.join(dataDirectory, 'modules/com.inductiveautomation.perspective/icons')

protectedLibraries = ['ignition', 'material', 'symbol_mimic', 'symbol_p&id', 'symbol_simple']

def getLibraries():
	'''Returns a list of library objects'''
	libraries = []
	contents = os.listdir(perspectiveIconFolder)
	for file in contents:
		split_tup = os.path.splitext(file)
		if split_tup[1] == ".svg":
			name = split_tup[0]
			
			libraries.append(Library(name))

	return libraries		

def createTempFile(event):
	'''Create temporary ZIP file'''
	if os.path.isdir(tempFolder) is False:
		os.mkdir(tempFolder)
	event.file.copyTo('%s/%s' % (tempFolder,event.file.name))

def deleteTempFiles():
	'''Delete temp files created for import'''
	for path, subdirs, files in os.walk(tempFolder, topdown=False):
		for file in files:
			filepath = os.path.join(path, file)
			os.remove(filepath)
		for dir in subdirs:
			dirpath = os.path.join(path, dir)
			os.rmdir(dirpath)
	
def createLibrary(libraryName):
	if libraryName in protectedLibraries:
		raise Exception("Library name is reserved")
	
	if libraryName == "" or libraryName is None:
		raise Exception("Library must have a name")
	
	existingLibraries = getLibraries()
	
	if libraryName in existingLibraries:
		raise Exception("Library must have a unique name")
	
	# Register appropriate xml namespaces
	ET.register_namespace('', "http://www.w3.org/2000/svg")
	ET.register_namespace('xlink', "http://www.w3.org/1999/xlink")
	
	# Format xml document
	xml = ET.Element('svg')
	defs = ET.SubElement(xml, 'defs')
	style = ET.SubElement(defs, 'style')
	style.text = ".icon { display: none } .icon:target { display: inline }"

	# Create tree from xml object
	newTree = ET.ElementTree(xml)
		
	# Get library path
	libraryName = libraryName+".svg"
	libraryPath = os.path.join(perspectiveIconFolder, libraryName)
	# Save xml to file
	newTree.write(libraryPath, encoding='utf-8',xml_declaration=True)

class Library:
	'''Library helper class'''
	def __init__(self, name):
		'''Initialize'''
		
		self.name = name
		self.path = self.getPathFromName(name)

	@staticmethod
	def getPathFromName(name):
		'''Get the absolute file path of a library by name'''
		libraryFile = name + ".svg"
		libraryPath = os.path.join(perspectiveIconFolder, libraryFile)
		
		return libraryPath

	@staticmethod
	def sendImportMessage(totalIcons = None, successfulIcons = None, failedIcons = None, log = None):
		
		payload = {}
		if totalIcons is not None:
			payload['totalIcons'] = totalIcons
		if successfulIcons is not None:
			payload['successfulIcons'] = successfulIcons
		if failedIcons is not None:
			payload['failedIcons'] = failedIcons
		if log is not None:
			payload['log'] = log
	
		system.perspective.sendMessage('iconImportStatus', payload)
		
	@staticmethod
	def extractZip(filepath):
		'''Extract zip to temp folder'''
		with ZipFile(filepath, 'r') as zip_ref:
			zip_ref.extractall(tempFolder)	
			
	@staticmethod
	def iterAttribFind(element, attrib, value=None):
		'''Returns all elements with given attribute'''
		if value is None:
				iterator = element.iterfind(".//*[@%s]" % attrib)
			
		else:
			iterator = element.iterfind(".//*[@%s='%s']" % (attrib, value))
		
		results = []
		for item in iterator:
			results.append(item)
		return results
			
	@staticmethod
	def getSVG(filepath, name):
		'''Return xml object for a SVG given a filepath and name'''
		tree = ET.parse(filepath)
		root = tree.getroot()
		attrib = root.attrib
		viewBox = attrib['viewBox']
		svg = ET.Element('svg', {"viewBox":viewBox})
	
		# Clear any conflicting attributes
		for element in root.iter():
			# List of attributes to remove
			attributes = ['class', 'id', 'data-name', 'fill']
			for attribute in attributes:
				element.attrib.pop(attribute, None)
		
		# Get top svg group
		svgGroup = root.find('g')
	
		# If no group exists create one
		if svgGroup is None:
			svgGroup = ET.Element('g')
			# Add elements to new group
			for element in root:
				svgGroup.append(element)
	
		# Add attributes to group
		svgGroup.attrib = {"class":"icon","id":name}
	
		# Add group to svg object
		svg.append(svgGroup)
	
		return svg
		
	@staticmethod
	def removeTag(tag, rootElement):
		'''Helper function to remove all tags of a specific name from a specified element'''
		for parent in rootElement.findall('.//%s/..' % tag ):
			for element in parent.findall('%s' % tag):
					parent.remove(element)

	def getName(self):
		'''Returns the name'''
		return self.name
		
	def getPath(self):
		'''Returns the abolute path'''
		return self.path

	def removeIcons(self, icons):
		'''Removes icons from library. Accepts a list of icon paths.'''
		libraryPath = self.path
				
		tree = ET.parse(libraryPath)
		root = tree.getroot()
		
		for icon in icons:
			name = icon.split('/')[1]
			iterator = root.iterfind(".//*[@id='%s']..." % name)
			for parent in iterator:
				root.remove(parent)
			
		tree.write(libraryPath, encoding='utf-8',xml_declaration=True)
		
	def getIcons(self):
		'''Returns all icon paths'''
		libraryPath = self.path
		
		tree = ET.parse(libraryPath)
		root = tree.getroot()
		
		icons = []
		
		attrib = 'id'
		
		elements = self.iterAttribFind(root, attrib)
		
		for element in elements:
			name = element.attrib['id']
			icons.append("%s/%s" % (self.name, name))
		
		return icons
	
	def rename(self, name):
		'''Rename library'''
		if name in protectedLibraries:
			raise Exception("Cannot change name of protected library")
			
		if name == "" or name is None:
			raise Exception("Library must have a name")
		
		existingLibraries = getLibraries()
		
		if name in existingLibraries:
			raise Exception("Library must have a unique name")
			
		existingPath = self.path
		newPath = self.getPathFromName(name)
		
		os.rename(existingPath, newPath)
			
	def delete(self):
		'''Delete library'''
		
		if self.name in protectedLibraries:
			raise Exception("Library name is reserved")
		
		libraryPath = self.path
		
		os.remove(libraryPath)
	
	def importFiles(self, overwrite=False):
		'''Imports all files in the temp directory'''
		log = "starting import into %s" % self.name
		self.sendImportMessage(log = log)
	
		# Unzip all .zip files in temp directory
		for path, subdirs, files in os.walk(tempFolder):
			for file in files:
				# Check for zip file type
				if file.endswith('.zip'):
					self.sendImportMessage(log = "extracting compressed file %s" % file)
					# Add root directory to file
					filepath = os.path.join(path, file)
					self.extractZip(filepath)
					self.sendImportMessage(log = "successfully extracted %s" % file)
		
		libraryPath = self.path
		
		tree = ET.parse(libraryPath)
		root = tree.getroot()
		
		filepaths = []
		i = 0
		for path, subdirs, files in os.walk(tempFolder):
			for file in files:
				i += 1
#				if i == 10:
#					break
				# Check for svg file type and uniqueness
				if file.endswith('.svg'):
					# Add root directory to file
					filepath = os.path.join(path, file)
					filepaths.append(filepath)
		
		totalIcons = len(filepaths)
		successfulIcons = 0
		failedIcons = 0
		self.sendImportMessage(totalIcons = totalIcons, log = "preparing to import %s icons" % totalIcons)
		
		for filepath in filepaths:
			newIcon = ET.parse(filepath).getroot()
			if newIcon.get('viewBox') is None:
				newIcon.set("viewbox", "0 0 24 24")
			name = os.path.basename(filepath).replace('.svg', '')
			self.sendImportMessage(log="adding icon: %s" %name)
			try:
				group = newIcon.find('g')
				
				if group is None:
					subElems = newIcon.findall('*')
					group = ET.Element('g')
					for elem in subElems:
						newIcon.remove(elem)
						group.append(elem)
					
					newIcon.append(group)
	
				group.set("class", "icon")
				group.set("id", name)
				
				root.append(newIcon)
				successfulIcons += 1
				self.sendImportMessage(successfulIcons = successfulIcons, log="icon %s added successfully" %name)
			except:
				failedIcons += 1
				self.sendImportMessage(failedIcons = failedIcons, log="icon %s failed" %name)


		# Write to file					
		self.sendImportMessage(log = "writing xml to file at path %s" % libraryPath)
		tree.write(libraryPath, encoding='utf-8',xml_declaration=True)
		
		# Remove all temp files
		self.sendImportMessage(log = "deleting temp files")
		deleteTempFiles()
			
		self.sendImportMessage(log = "import complete")		
	
	
	
	
	
	
	
	
	
	
	

